/*
 * PDFlib PLOP DS:
 * generate a document in memory with PDFlib and sign it with PLOP DS
 */

#include <iostream>

#include "pdflib.hpp"
#include "plop.hpp"

using namespace std;
using namespace pdflib;

int
main()
{
    int font;
    const char *contents = NULL;
    long size;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    wstring out_filename(L"hellosign.pdf");

    /* Basic signature options */
    static const wstring sign_opts =
        L"ltv=try "          /* use ltv=full to ensure LTV status */
    #if defined(USE_PKCS11)
        L"engine=pkcs#11 digitalid={filename=cryptoki.dll}"
    #else
        L"engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo}"
    #endif
        ;

    /* ---------- Step 1: generate document in memory with PDFlib ---------- */

    try {
        PDFlib p;

        /* create a new PDF document in memory */
    p.begin_document(L"", L"");

    p.set_info(L"Creator", L"hellosign.c");
    p.set_info(L"Author", L"Rainer Pl\u00f6ckl");
    p.set_info(L"Title", L"Hello, world/signed (C)!");

    font = p.load_font(L"Helvetica-Bold", L"host", L"");

    p.begin_page_ext(a4_width, a4_height, L"");

    p.setfont(font, 24);
    p.set_text_pos(50, 700);
    p.show(L"Hello, world/signed!");
    p.continue_text(L"(says C)");
    p.end_page_ext(L"");

    p.end_document(L"");

    /* fetch the memory buffer containing the PDF document */
    contents = p.get_buffer(&size);
    }
    catch (PDFlib::Exception &ex) {
        wcerr << L"PDFlib exception occurred in hellosign sample:" << endl;
        wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
        << L": " << ex.get_errmsg() << endl;
        
        return 98;
    }

    /* ------------------ Step 2: sign the document ----------------- */
    
    try {
    int doc;
    PLOP plop;
    wostringstream optlist;

    optlist << L"searchpath={" << searchpath << L"}";

    plop.set_option(optlist.str());

    /* Create a virtual PVF file from the document data in memory */
    plop.create_pvf(L"/pvf/hello.pdf", contents, size, L"");

    /* open input document from PVF file */
    if ((doc = plop.open_document(L"/pvf/hello.pdf", L"")) == -1)
    {
            wcerr << L"Error: " << plop.get_apiname() << L": "
                    << plop.get_errmsg() << endl;
        return 99;
    }

        if (plop.prepare_signature(sign_opts) == -1)
        {
            wcerr << L"Error: " << plop.get_apiname() << L": "
                    << plop.get_errmsg() << endl;
            return 2;
        }

    /* sign the output file */
        optlist.str(L"");
        optlist << L"input=" << doc;
    if (plop.create_document(out_filename, optlist.str()) == -1)
    {
            wcerr << L"Error: " << plop.get_apiname() << L": "
                    << plop.get_errmsg() << endl;
        return 99;
    }

        /* close the input document */
        plop.close_document(doc, L"");

        /* delete PVF file containing input document */
        if (plop.delete_pvf(L"/pvf/hello.pdf") == -1)
    {
            wcerr << L"Error: " << plop.get_apiname() << L": "
                    << plop.get_errmsg() << endl;
        return 99;
    }

    }
    catch (PLOP::Exception &ex) {
    wcerr << L"PLOP exception occurred in hellosign:" << endl;
    wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
        << L": " << ex.get_errmsg() << endl;
    return 99;
    }

    return 0;
}
