# PDFlib PLOP DS:
# Demonstrate mass signature with optional PKCS#11 session handling
#

from sys import *
from PDFlib.PLOP import *

# Basic signature options 

# use ltv=full to ensure LTV status
# for  use with PKCS11
# "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
sign_opts = \
    "ltv=try " \
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
    

# This is where input files live. Adjust as necessary.
searchpath = "../data"

# Set to 1 to enable PKCS11 session handling
PKCS11 = 0

if len(argv) < 2:
    raise Exception("usage: multisign <infile> ..\n." )

plop = None
try:
    # create a new PLOP object
    plop = PLOP()

    optlist = "searchpath={%s}" % searchpath
    plop.set_option(optlist)

    if (plop.prepare_signature(sign_opts) == -1):
        raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                 plop.get_errmsg()))

    for i in range (1, len(argv), 1):
        print("Signing '%s' ..." % argv[i])
        doc = plop.open_document(argv[i], "")
        if (doc == -1):
            print("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))
            continue

        signed_name = argv[i] + "-signed.pdf"
        optlist = "input={%d}" % doc

        if (plop.create_document(signed_name, optlist) == -1):
            print("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))

        if (PKCS11 == 1):
            # The PKCS#11 session may have expired;
            # restart and try again.
            if (plop.prepare_signature(sign_opts) == -1):
                raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                         plop.get_errmsg()))

            if (plop.create_document(signed_name, optlist) == -1):
                raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                         plop.get_errmsg()))

        plop.close_document(doc, "")

except PLOPException as ex:
    print("PDFlib PLOP exception occurred:")
    print("[%d] %s: %s" % (ex.errnum, ex.apiname, ex.errmsg))

except Exception as ex:
    print(ex)

finally:
    if plop:
        plop.delete()
